package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ui.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.ArrayAssigner.*;
import static com.goldencode.p2j.util.character.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in main.p).
 */
public class Main
{
   logical progStarted = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "prog-started", new logical(false));

   character uName = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "u-name", new character(""));

   character screenTitle = SharedVariableManager.addVariable("screen-title", new character(""));

   integer totalFloorNo = SharedVariableManager.addVariable("total-floor-no", new integer(4));

   character[] roomTypes = SharedVariableManager.addVariable("room-types", new character[5]);

   character[] resState = SharedVariableManager.addVariable("res-state", new character[4]);

   character[] resStateCh = SharedVariableManager.addVariable("res-state-ch", new character[4]);

   integer resStatePlaced = SharedVariableManager.addVariable("res-state-placed", new integer(1));

   integer resStateCanceled = SharedVariableManager.addVariable("res-state-canceled", new integer(2));

   integer resStateCheckedIn = SharedVariableManager.addVariable("res-state-checked-in", new integer(3));

   integer resStateCheckedOut = SharedVariableManager.addVariable("res-state-checked-out", new integer(4));

   character[] curProg = TypeFactory.characterExtent(10);

   integer curProgIdx = TypeFactory.integer();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in main.p).
    */
   public void execute()
   {
      character startProg = TypeFactory.character("menu.p");
      logical vStop = UndoableFactory.logical();
      
      externalProcedure(Main.this, new Block((Body) () -> 
      {
         progStarted.assign(true);
         assignSingle(roomTypes, 1, "1 bed king");
         assignSingle(roomTypes, 2, "1 bed queen");
         assignSingle(roomTypes, 3, "2 beds");
         assignSingle(roomTypes, 4, "1 room apt");
         assignSingle(roomTypes, 5, "2 room apt");
         assignSingle(resState, 1, "Placed");
         assignSingle(resState, 2, "Canceled");
         assignSingle(resState, 3, "Checked in");
         assignSingle(resState, 4, "Checked out");
         assignSingle(resStateCh, 1, "P");
         assignSingle(resStateCh, 2, "C");
         assignSingle(resStateCh, 3, "I");
         assignSingle(resStateCh, 4, "O");
         ProcedureManager.addSuperProcedure(thisProcedure());
         NamedEventManager.subscribeAnywhere(thisProcedure(), "push-program");
         NamedEventManager.subscribeAnywhere(thisProcedure(), "pop-program");
         NamedEventManager.publish("push-program", thisProcedure(), "I", new Object[]
         {
            startProg
         });
         
         OnPhrase[] onPhrase0 = new OnPhrase[]
         {
            new OnPhrase(Condition.ENDKEY, Action.RETRY, "loopLabel1"),
            new OnPhrase(Condition.STOP, Action.RETRY, "loopLabel1"),
            new OnPhrase(Condition.ERROR, Action.RETRY, "loopLabel1"),
            new OnPhrase(Condition.QUIT, Action.LEAVE, "loopLabel1")
         };
         
         repeat("loopLabel1", onPhrase0, new Block((Body) () -> 
         {
            hideAll(true);
            
            hideMessage(true);
            
            vStop.assign(true);
            progStarted.assign(false);
            
            OnPhrase[] onPhrase1 = new OnPhrase[]
            {
               new OnPhrase(Condition.STOP, Action.LEAVE, "blockLabel0")
            };
            
            doBlock(TransactionType.SUB, "blockLabel0", onPhrase1, new Block((Body) () -> 
            {
               OnPhrase[] onPhrase2 = new OnPhrase[]
               {
                  new OnPhrase(Condition.ERROR, Action.LEAVE, "blockLabel1"),
                  new OnPhrase(Condition.ENDKEY, Action.LEAVE, "blockLabel1")
               };
               
               doBlock(TransactionType.SUB, "blockLabel1", onPhrase2, new Block((Body) () -> 
               {
                  /* ignore error/endkey, as the program was able to be run */
                  ControlFlowOps.invoke((peekProgram()).toStringMessage());
               }));
               
               vStop.assign(false);
            }));
            
            if ((vStop).booleanValue())
            {
               if ((progStarted).booleanValue())
               {
                  /* re-init the program */
                  assignUnknown(curProg);
                  curProgIdx.assign(0);
                  NamedEventManager.publish("push-program", thisProcedure(), "I", new Object[]
                  {
                     startProg
                  });
               }
               else
               {
                  messageBox(new Object[]
                  {
                     "Program",
                     peekProgram(),
                     "could not be executed."
                  }, ALERT_MESSAGE, BTN_OK, (String) null);
                  
                  hideMessage(true);
                  
                  hideAll(true);
                  
                  statusDefault();
                  putScreen(fill(" ", 78), plus(getScreenLines(), 1), 1, new ColorSpec(ColorSpec.COLOR_NORMAL));
                  putScreen(fill(" ", 78), plus(getScreenLines(), 2), 1, new ColorSpec(ColorSpec.COLOR_NORMAL));
                  next("loopLabel1");
               }
            }
         }));
      }));
   }

   /**
    * TODO: determine start-prog based on rights/entry-point/authenticated user<br>
    * TODO: help
    */
   public void pushProgram(final character _prog)
   {
      character prog = UndoableFactory.initInput(_prog);
      
      internalProcedure(new Block((Body) () -> 
      {
         curProgIdx.assign(plus(curProgIdx, 1));
         assignSingle(curProg, curProgIdx, prog);
      }));
   }

   public void popProgram()
   {
      internalProcedure(new Block((Body) () -> 
      {
         assignSingleUnknown(curProg, curProgIdx);
         curProgIdx.assign(minus(curProgIdx, 1));
      }));
   }

   public character peekProgram()
   {
      return function("peek-program", character.class, new Block((Body) () -> 
      {
         returnNormal(subscript(curProg, curProgIdx));
      }));
   }

   public void setRoomTypesCombo(final handle _h)
   {
      handle h = UndoableFactory.initInput(_h);
      character ch = UndoableFactory.character();
      integer i = UndoableFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         loopLabel0:
         for (i.assign(1); _isLessThanOrEqual(i, roomTypes.length); i.increment())
         {
            ch.assign(concat(ch, (_isEqual(i, 1) ? new character("") : new character(",")), subscript(roomTypes, i), ",", valueOf(i)));
         }
         
         h.unwrapCommonListWidget().setListItemPairs(ch);
      }));
   }
}
